<?php
namespace kwy\third_interface;
/**
 *
 * Created by PhpStorm.
 * User: roc
 * Date: 2019/12/6
 * Time: 13:39
 */


use kwy\util\http\CurlHttp;

/**
 * 购阿购接口类
 * Class GooAGoo
 * @package kwy\third_interface
 */
class GooAGoo
{
    private $appId = "";    //数衍平台分发给开发者应用的appId
    private $appKey = "";    //数衍平台分发给开发者应用的appKey
    private $secretKey = ""; //数衍平台应用的秘钥
    private $channelId = ""; //渠道号,数衍平台提供

    /**
     * @var CurlHttp
     */
    private $httpClient;

    /**
     * 设置数衍平台分发给开发者应用的appId
     * @author roc
     * @param $appId
     * @return $this
     */
    public function setAppId($appId){
        $this->appId = $appId;
        return $this;
    }

    /**
     * 获取数衍平台分发给开发者应用的appId
     * @author roc
     * @return string
     */
    public function getAppId(){
        return $this->appId;
    }

    /**
     * 设置数衍平台分发给开发者应用的appKey
     * @author roc
     * @param $appKey
     * @return $this
     */
    public function setAppKey($appKey){
        $this->appKey = $appKey;
        return $this;
    }

    /**
     * 获取数衍平台分发给开发者应用的appKey
     * @author roc
     * @return string
     */
    public function getAppKey(){
        return $this->appKey;
    }

    /**
     * 设置数衍平台应用的秘钥
     * @author roc
     * @param $secretKey
     * @return $this
     */
    public function setSecretKey($secretKey){
        $this->secretKey = $secretKey;
        return $this;
    }

    /**
     * 获取数衍平台应用的秘钥
     * @author roc
     * @return string
     */
    public function getSecretKey(){
        return $this->secretKey;
    }

    /**
     * 设置渠道号
     * @author roc
     * @datetime:2019/12/13 16:22
     * @param $channelId
     * @return $this
     */
    public function setChannelId($channelId){
        $this->channelId = $channelId;
        return $this;
    }

    /**
     * 获取渠道号
     * @author roc
     * @datetime:2019/12/13 16:22
     * @return string
     */
    public function getChannelId(){
        return $this->channelId;
    }

    /**
     * 订单同步
     * @author roc
     * @datetime:2019/12/13 10:11
     * @param string $saleTime 销售时间
     * @param string $out_no 第三方账单唯一标识
     * @param string $receivableAmount 应收金额
     * @param double $totalNum 商品数量,支持小数
     * @param string $totalFee 总金额
     * @param string $out_shop_id 此处ID为第三方商户唯一标识
     * @param string $couponAmount 优惠金额,非必传
     * @param string $paidAmount 实收金额，保留两位小数；数衍平台重打单或重解析时会发生变化；非必传
     * @param string $discountAmount 折扣金额，保留两位小数；数衍平台重打单或重解析时会发生变化;非必传
     * @param array $productList 商品数据
     *  Ex:
     *              [
     *                  "name" => "益达口香糖",
     *                  "itemserial" => "6923450657438",
     *                  "price" => 0.9,
     *                  "totalnum" => 5,
     *                  "totalprice" => 4.5
     *              ]
     * @return bool
     * @throws \Exception
     */
    public function syncOrder($saleTime,$out_no,$receivableAmount,$totalNum,$totalFee,$out_shop_id,$couponAmount='',$paidAmount='',$discountAmount='',$productList=[]){
        $url = "http://api.gooagoo.com/oapi/rest";

        $data = [
            "saleTime" => $saleTime,
            "thirdPartyOrderNo" => $out_no,
            "receivableAmount" => $receivableAmount,
            "totalNum" => $totalNum,
            "totalFee" => $totalFee,
            "shopEntityId" => $out_shop_id,
            "channelCode" => $this->channelId,
        ];

        //优惠金额
        if(!empty($couponAmount)){
            $data["couponAmount"] = $couponAmount;
        }
        //实收金额
        if(!empty($paidAmount)){
            $data["paidAmount"] = $paidAmount;
        }
        //折扣金额
        if(!empty($discountAmount)){
            $data["discountAmount"] = $discountAmount;
        }
        //商品数据
        if(!empty($productList)){
            $data["goodsDetails"] = $productList;
        }

        $request_data = [
            "appId" => $this->appId,    //数衍平台分发给开发者应用的ID，在开放平台创建应用时自动分配
            "method" => "gogo.open.auto.routing",    //固定值，gogo.open.auto.routing
            "timestamp" => date("YmdHis"), //发送请求的时间，格式 “yyyyMMDDHHmmss”
            "messageFormat" => "json",    //接口报文格式，目前仅支持json格式
            "appKey" => $this->appKey,  //数衍平台分发给开发者应用的appKey
            "v" => "1.0",  //接口版本，固定为：1.0
            "signMethod" => "MD5",    //签名算法类型，目前仅支持MD5
            "lowerMethod" => "com.gooagoo.exportbills",    //业务接口名称
            "data" => json_encode($data),  //业务请求参数的集合，最大长度不限，除公共参数外所有请求参数都必须放在这个参数中传递
        ];
        $request_data['sign'] = $this->getSign($request_data);

        $response = $this->getHttpClient()
//            ->setProxy('127.0.0.1:8888')
            ->setUrl($url)
            ->setContentType("application/x-www-form-urlencoded;charset=utf-8")
            ->post(http_build_query($request_data));

        if(empty($response)) throw new \Exception("购阿购没有返回数据");

        $result = json_decode($response,true);
        if(empty($result)) throw new \Exception("购阿购返回数据异常");

        if($result["rescode"] != "OPEN_SUCCESS"){
            throw new \Exception("购阿购账单同步失败：".$result["resmsg"]);
        }
        return $result["data"];
    }

    /**
     * 获取签名
     * @author roc
     * @datetime:2019/12/13 10:11
     * @param $data
     * @return string
     */
    private function getSign($data){
        ksort($data);
        $str = "";
//        var_dump($data);
        foreach($data as $k => $v){
            $str .= $k.'='.$v.'&';
//            echo $str."\n";
        }
        $str .= "key=".$this->secretKey;
        return strtoupper(md5($str));
    }

    /**
     * 设置代理服务器
     * @author roc
     * @datetime:2019/12/16 14:19
     * @param string $proxy 代理服务器地址
     * @return CurlHttp
     */
    public function setProxy($proxy = "127.0.0.1:8888"){
        $client = $this->getHttpClient();
        $client->setProxy($proxy);
        return $client;
    }

    /**
     * 获取请求客户端
     * @author roc
     * @datetime:2019/12/13 18:12
     * @return CurlHttp
     */
    private function getHttpClient(){
        if (empty($this->httpClient)){
            $this->httpClient = new CurlHttp();
            //$this->httpClient->setProxy('127.0.0.1:8888');
            $this->httpClient->setContentType('application/json');
        }
        return $this->httpClient;
    }
}